@@ -70,10 +70,10 @@
 
 (defcustom lsp-ui-peek-fontify 'on-demand
   "Whether to fontify chunks of code (use semantics colors).
-WARNING: 'always can heavily slow the processing when
+WARNING: `always can heavily slow the processing when
 `lsp-ui-peek-expand-function' expands more than 1 file.
 It is recommended to keep the default value of `lsp-ui-peek-expand-function'
-when this variable is set to 'always."
+when this variable is set to `always."
   :type '(choice (const :tag "Never" never)
                  (const :tag "On demand" on-demand)
                  (const :tag "Always" always))
@@ -147,7 +147,7 @@ The function takes one parameter: a list of cons where the car is the
 filename and the cdr is the number of references in that file.
 It should returns a list of filenames to expand.
 WARNING: If you change this variable and expand more than 1 file, it is
-recommended to set `lsp-ui-peek-fontify' to 'never or 'on-demand, otherwise it
+recommended to set `lsp-ui-peek-fontify' to `never or `on-demand, otherwise it
 will cause performances issues.")
 
 (defvar-local lsp-ui-peek--overlay nil)
@@ -319,7 +319,12 @@ XREFS is a list of references/definitions."
                                 'face 'lsp-ui-peek-filename
                                 'file filename
                                 'xrefs xrefs)
-                    (propertize " " 'display `(space :align-to (- right-fringe ,(1+ (length len-str)))))
+                    (propertize " " 'display `(space :align-to (- right-fringe
+                                                                  ;; Account for Emacs TTY's window divider
+                                                                  ;; Without this leeway, the reference count
+                                                                  ;; string goes to next line - impairs readability
+                                                                  ,(if (display-graphic-p) 0 1)
+                                                                  ,(1+ (length len-str)))))
                     (propertize len-str 'face 'lsp-ui-peek-filename))
             lsp-ui-peek--list)))
   (setq lsp-ui-peek--list (nreverse lsp-ui-peek--list))
@@ -544,6 +549,7 @@ XREFS is a list of references/definitions."
     (define-key map (kbd "p") 'lsp-ui-peek--select-prev)
     (define-key map (kbd "<up>") 'lsp-ui-peek--select-prev)
     (define-key map (kbd "TAB") 'lsp-ui-peek--toggle-file)
+    (define-key map (kbd "<tab>") 'lsp-ui-peek--toggle-file)
     (define-key map (kbd "q") 'lsp-ui-peek--abort)
     (define-key map (kbd "RET") 'lsp-ui-peek--goto-xref)
     (define-key map (kbd "M-RET") 'lsp-ui-peek--goto-xref-other-window)
@@ -629,8 +635,8 @@ EXTRA is a plist of extra parameters."
                            (append extra (lsp--text-document-position-params))))
 
 (defun lsp-ui-peek--extract-chunk-from-buffer (pos start end)
-  "Return the chunk of code pointed to by POS (a Position object) in the current buffer.
-START and END are delimiters."
+  "Return the chunk of code pointed to by POS (a Position object) in the
+current buffer.  START and END are delimiters."
   (let* ((point (lsp--position-to-point pos))
          (inhibit-field-text-motion t)
          (line-start (1+ (- 1 (/ lsp-ui-peek-peek-height 2))))
@@ -720,7 +726,10 @@ references.  The function returns a list of `ls-xref-item'."
 Returns item(s)."
   (-when-let* ((locs (lsp-request method params))
                (locs (if (listp locs)
-                         locs
+                         (if (symbolp (car locs))
+                             ;; A single plist was returned
+                             (list locs)
+                           locs)
                        (if (vectorp locs)
                            (append locs nil)
                          (list locs)))))
