@@ -77,7 +77,7 @@
   :type 'boolean
   :group 'lsp-ui-sideline)
 
-(defcustom lsp-ui-sideline-show-code-actions t
+(defcustom lsp-ui-sideline-show-code-actions nil
   "Whether to show code actions in sideline."
   :type 'boolean
   :group 'lsp-ui-sideline)
@@ -299,13 +299,13 @@ MARKED-STRING is the string returned by `lsp-ui-sideline--extract-info'."
   (+ (apply '+ lengths)
      (if (display-graphic-p) 1 2)))
 
-(defun lsp-ui-sideline--compute-height nil
+(defun lsp-ui-sideline--compute-height ()
   "Return a fixed size for text in sideline."
   (if (null text-scale-mode-remapping)
       '(height 1)
     ;; Readjust height when text-scale-mode is used
     (list 'height
-          (/ 1 (or (plist-get (cdr text-scale-mode-remapping) :height)
+          (/ 1 (or (plist-get (cdar text-scale-mode-remapping) :height)
                    1)))))
 
 (defun lsp-ui-sideline--make-display-string (info symbol current)
@@ -325,8 +325,8 @@ CURRENT is non-nil when the point is on the symbol."
      (propertize str 'display (lsp-ui-sideline--compute-height)))))
 
 (defun lsp-ui-sideline--check-duplicate (symbol info)
-  "Check if there's already a SYMBOL containing INFO, unless `lsp-ui-sideline-ignore-duplicate'
-is set to t."
+  "Check if there's already a SYMBOL containing INFO, unless
+`lsp-ui-sideline-ignore-duplicate' is set to t."
   (not (when lsp-ui-sideline-ignore-duplicate
          (--any (and (string= (overlay-get it 'symbol) symbol)
                      (string= (overlay-get it 'info) info))
@@ -435,7 +435,8 @@ is set to t."
           (lsp-ui-sideline--toggle-current ov nil))))))
 
 (defun lsp-ui-sideline--split-long-lines (lines)
-  "Fill LINES so that they are not longer than `lsp-ui-sideline-diagnostic-max-line-length' characters."
+  "Fill LINES so that they are not longer than
+`lsp-ui-sideline-diagnostic-max-line-length' characters."
   (cl-mapcan (lambda (line)
                (if (< (length line) lsp-ui-sideline-diagnostic-max-line-length)
                    (list line)
@@ -449,7 +450,8 @@ is set to t."
 (defun lsp-ui-sideline--diagnostics (buffer bol eol)
   "Show diagnostics belonging to the current line.
 Loop over flycheck errors with `flycheck-overlay-errors-in'.
-Find appropriate position for sideline overlays with `lsp-ui-sideline--find-line'.
+Find appropriate position for sideline overlays with
+`lsp-ui-sideline--find-line'.
 Push sideline overlays on `lsp-ui-sideline--ovs'."
   (when (and (bound-and-true-p flycheck-mode)
              (bound-and-true-p lsp-ui-sideline-mode)
